;****************************************************************
;*  Name    : MIDIDESC_3407.inc                                 *
;*  Author  : Gabi MIHAILA                                      *
;*  Notice  : Copyright (c) 2009 Orbtronics                     *
;*          : All Rights Reserved                               *
;*  Date    : 04-Oct-09                                         *
;*  Version : 2.0                                               *
;*  Notes   : USB Audio Class - MIDI section                    *
;*          : minimal MIDI Streaming Interface Descriptors      *
;*          : To be used with PDS compiler version 3.4.0.7      *
;****************************************************************
;*     : Please read the:                                       *
;*     : Device Class Definition for Midi Devices               *
;*     : available from:                                        *
;*     : http://www.usb.org/developers/devclass_docs/midi10.pdf *
;****************************************************************   

#define  __EP0_BUFF_SIZE         8         ; 8, 16, 32, or 64
#define  __MAX_NUM_INT           1         ; For tracking Alternate Setting
#define  __MAX_EP_NUMBER         3         ; UEP3
#define  __NUM_CONFIGURATIONS    1
#define  __NUM_INTERFACES        2

#define  __MODE_PP               _PPBM0
#define  __UCFG_VAL              _PUEN | _TRINT | _FS | __MODE_PP  ; Set bus for Full Speed

#define __USB_USE_MIDI           ; Inform the rest of the library subroutines that we're using a MIDI interface
                                 ; and also for not re-inventing the wheel :)
#define __USB_USE_CDC            ; we will use the variables and some declares needed for USB Standard Class

; MIDI Streaming Interface Class Code
#define __AUDIO_INTF            0x01
; MIDI Streaming Interface Class SubClass Codes
#define __MIDI_Streaming        0x03
#define __MIDI_IN_JACK          0x02
#define __MIDI_OUT_JACK         0x03
;#define __ELEMENT               0x04  ; for later use
;#define __JACK_TYPE_UNDEFINED   0x00  ; for later use
#define __JACK_EMBEDDED         0x01
#define __JACK_EXTERNAL         0x02
#define __MS_GENERAL            0x01

; // CDC declares stays so we do not mess with internal variables and their specific activation/usage
; // And BTW - MIDI Streaming does not require a special class after all , just standard USB :)
#define __CDC_COMM_INTF_ID        0
#define __CDC_COMM_UEP            UEP2
#define __CDC_INT_BD_IN           __ep2Bi
#define __CDC_INT_EP_SIZE         8
#define __CDC_DATA_INTF_ID        1
#define __CDC_DATA_UEP            UEP3
#define __CDC_BULK_BD_OUT         __ep3Bo
#define __CDC_BULK_OUT_EP_SIZE    64       ; 64 byte output buffer.
#define __CDC_BULK_BD_IN          __ep3Bi
#define __CDC_BULK_IN_EP_SIZE     64       ; 64 byte input buffer

;--------------------------------
#define __VID_Value 0x04D8                 ; VID (Vendor Id)
#define __PID_Value 0x0056                 ; PID (Product Id)
#define __MAX_POWER 40                     ; Maximum power consumption (in mA)
;
; Vendor Name String
#define __VdrNmeStr 'M', 0, 'i', 0, 'c', 0, 'r', 0, 'o', 0, 'c', 0, 'h', 0, 'i', 0, 'p', 0
; Device Description String
#define __DvcDscStr 'M', 0, 'I', 0, 'D', 0, 'I', 0, ' ', 0, 'S', 0, 't', 0, 'r', 0, 'e', 0, 'a', 0, 'm', 0, 'i', 0, 'n', 0, 'g', 0
; Version String
#define __VerStr    'V', 0, '2', 0, 'B', 0, 'e', 0, 't', 0, 'a', 0

; ------------------------------------------------------------------------------
; This table is polled by the host immediately after USB Reset has been released.
; These fields are application dependent to be modified to meet your specifications.
;
__DeviceDescriptor
  retlw  (__EndDeviceDescriptor - __DeviceDescriptor) / 2  ; Length of this descriptor
  retlw  __DSC_DEV                    ; bDescType Device
  dt     0x10, 0x01                   ; bcd USB Revision 1.10 low byte, high byte
  retlw  0                 ; bDeviceClass
  retlw  0x00                         ; bDeviceSubClass
  retlw  0x00                         ; bDeviceProtocol
  retlw  __EP0_BUFF_SIZE              ; bMaxPacketSize for EP0
  dt     Low(__VID_Value), High(__VID_Value)   ; Vendor Id (VID) low byte, high byte
  dt     Low(__PID_Value), High(__PID_Value)   ; Product ID (PID) low byte, high byte
  dt     0x02, 0x00                   ; bcd Device Device release number low byte, high byte
  retlw  0x01                         ; iManufacturer Manufacturer string index
  retlw  0x02                         ; iProduct Product string index
  retlw  0x00                         ; iSerialNumber Device serial number string index
  retlw  __NUM_CONFIGURATIONS         ; bNumConfigurations
__EndDeviceDescriptor

; ------------------------------------------------------------------------------
; This table is retrieved by the host after the address has been set.
; This table defines the configurations available for the device.
; See section 9.6.2 of the Rev 1.0 USB specification (page 184).
; These fields are application dependent to be modified to meet your specifications.
;
; Configuration pointer table
__USB_CD_Ptr
__Configs
  db  Low ___Config1, High ___Config1
  db  Low ___Config1, High ___Config1
; Configuration Descriptor
___Config1
  retlw  9                            ; Length of this descriptor
  retlw  __DSC_CFG                    ; bDescType Configuration
__Config1Len
  retlw  Low ((__EndConfig1 - ___Config1) / 2)    ; Length of this configuration
  retlw  High ((__EndConfig1 - ___Config1) / 2)
  retlw  __NUM_INTERFACES             ; Number of interfaces in this configuration
  retlw  0x01                         ; Configuration Value
  retlw  0x00                         ; Index for this configuration
  retlw  0x80                         ; bmAttributes
  retlw  (__MAX_POWER * 2)            ; Max power consumption (2X mA)

; Midi Adapter Standard AC Interface Descriptor
;----------------------------------------------
	retlw	9                   ; bLength
	retlw	4					; Interface Descriptor
	retlw	0					; interface number
	retlw 	0					; alternate setting
	retlw	0					; number of endpoints
	retlw	1					; btInterface Class Audio
	retlw	1					; btInterface Subclass Audio Control
	retlw	0					; btInterface Protocol (unused)
	retlw	0					; interface (unused)
; Midi Adapter Class Specific AC Interface Descriptor
;-----------------------------------------------------
	retlw	9					; bLength
	retlw	__CS_INTERFACE		; descriptor type CS INTERFACE
	retlw	1					; header subtype
	dt 		0x00, 0x01			; bcd ADC - revision of Class
	dt		0x09, 0x00			; total size of class specific descriptor
	retlw	1					; number of Streming Interfaces
	retlw	1					; MIDIStreaming interface 1 belongs to this AudioControl interface	   
; MIDI Adapter Standard MS Interface Descriptor
;----------------------------------------------
	retlw	9					; descriptor size
	retlw	4					; descriptor type INTERFACE Descriptor
	retlw	1					; Interface Number
	retlw	0					; alternate setting
	retlw	2					; 2 endpoints
	retlw	__AUDIO_INTF		; interface class Audio
	retlw	__MIDI_Streaming	; interface subclass Midi Streaming
	retlw	0					; interface protocol (not used)
	retlw	0					; interface (not used)	
; MIDI Streaming Class-Specific Interface Descriptors
;-------------------------------------------------------
	retlw	7											; Length  
	retlw	__CS_INTERFACE								; bDscType
	retlw	1 								            ; MIDI STreaming sub class release number in BCD
	dt		0x00, 0x01									; bcd low byte, high byte
    dt      0x41, 0x00                                  ; total number of bytes returned    
; MIDI Adapter MIDI IN Jack Descriptor (Embedded)
;-----------------------------------------------
	retlw	6											; Length  
	retlw	__CS_INTERFACE								; bDscType
	retlw	__MIDI_IN_JACK								; MIDI_IN_JACK Descriptor Sub type
	retlw	__JACK_EMBEDDED								; MIDI Jack Type EMBEDDED or EXTERNAL
    retlw   1                                           ; MIDI JACK ID
    retlw   0                                           ; Index of a string descriptor for this jack
; MIDI Adapter MIDI IN Jack Descriptor (External)
;-------------------------------------------------
	retlw	6											; Length  
	retlw	__CS_INTERFACE								; bDscType
	retlw	__MIDI_IN_JACK								; MIDI_IN_JACK Descriptor Sub type
	retlw	__JACK_EXTERNAL								; MIDI Jack Type EMBEDDED or EXTERNAL
	retlw	2 							                ; MIDI JACK ID
    retlw   0                                           ; Index of a string descriptor For this jack
; MIDI Adapter MIDI OUT Jack Descriptor (Embedded)
;---------------------------------------------------
	retlw	9											; Length  
	retlw	__CS_INTERFACE								; bDscType
	retlw	__MIDI_OUT_JACK							    ; MIDI_OUT_JACK Descriptor Sub type
	retlw	__JACK_EMBEDDED								; MIDI Jack Type EMBEDDED or EXTERNAL
	retlw	3 							                ; MIDI JACK ID
    retlw   1                                           ; Number of Input pins on this Jack
    retlw   2                                           ; ID of the Entity to which this pin is connected
    retlw   1                                           ; Output pin number of the Entity to which this pin is connected
    retlw   0                                           ; Unused    
; MIDI Adapter MIDI OUT Jack Descriptor (External)
;-------------------------------------------------
    retlw	9											; Length  
	retlw	__CS_INTERFACE								; bDscType
	retlw	__MIDI_OUT_JACK							    ; MIDI_OUT_JACK Descriptor Sub type
	retlw	__JACK_EXTERNAL								; MIDI Jack Type EMBEDDED or EXTERNAL
	retlw	4 							                ; MIDI JACK ID
    retlw   1                                           ; Number of Input pins on this Jack
    retlw   1                                           ; ID of the Entity To which this pin is connected
    retlw   1                                           ; Output pin number of the Entity to which this pin is connected
    retlw   0                                           ; Unused
; MIDI ADAPTER STANDARD BULK OUT Endpoint Descriptor
;----------------------------------------------------
	retlw	9											; Length
	retlw	__DSC_EP									; bDscType
	retlw	_EP03_OUT							        ; OUT Endpoint used
	retlw	2 											; use BULK Transfer
	dt      0x40, 0x00                                  ; 64 bytes per packet
    retlw	0 						                    ; Ignored for BULK. Set to 0.
	retlw	0                                           ; Not used
	retlw	0 											; Not used    
; MIDI ADAPTER STANDARD CLASS SPECIFIC BULK OUT Endpoint Descriptor    
;------------------------------------------------------------------ 
    retlw   5                                           ; Length
    retlw   __CS_ENDPOINT                               ; Class Endpoint descriptor
    retlw   __MS_GENERAL                                ; Descriptor SubType
    retlw   1                                           ; number of embedded MIDI IN Jacks
    retlw   1                                           ; ID of the EMBEDDED MIDI IN Jack
; MIDI ADAPTER STANDARD BULK IN Endpoint Descriptor    
;----------------------------------------------------
    retlw	9											; Length  
	retlw	__DSC_EP									; bDscType
	retlw	_EP03_IN						            ; IN Endpoint used
	retlw	2 											; use BULK Transfer
	dt     0x40, 0x00                                   ; 64 bytes per packet
    retlw	0 						                    ; Ignored for BULK. Set to 0.
	retlw	0                                           ; Not used
	retlw	0 											; Not used
; MIDI ADAPTER CLASS SPECIFIC BULK IN Endpoint Descriptor    
;----------------------------------------------------
    retlw   5                                           ; Length
    retlw   __CS_ENDPOINT                               ; Class Endpoint descriptor
    retlw   __MS_GENERAL                                ; Descriptor SubType
    retlw   1                                           ; number of EMBEDDED MIDI OUT Jacks
    retlw   3                                           ; ID of the EMBEDDED MIDI OUT Jack
__EndConfig1



; String pointer table
__USB_SD_Ptr
__Strings
  db  Low __String0, High __String0
  db  Low __String1, High __String1
  db  Low __String2, High __String2
  db  Low __String3, High __String3
  db  Low __String4, High __String4

__String0
  retlw   (__String1 - __String0) / 2  ; Length of string
  retlw   __DSC_STR                    ; Descriptor type 3
  dt      0x09, 0x04                   ; Language ID (0x0409 as defined by Microsoft)
__String1
  retlw   (__String2 - __String1) / 2
  retlw   __DSC_STR
  dt __VdrNmeStr
__String2
  retlw   (__String3 - __String2) / 2
  retlw   __DSC_STR
  dt __DvcDscStr
__String3
  retlw   (__String4 - __String3) / 2
  retlw   __DSC_STR
  dt __VerStr
__String4
  retlw   (__String5 - __String4) / 2
  retlw   __DSC_STR
  dt      'C', 0, 'f', 0, 'g', 0, '1', 0
__String5
