'****************************************************************
'*  Name    : USB_MD_RX.BAS                                     *
'*  Author  : Gabi MIHAILA                                      *
'*  Notice  : Copyright (c) 2009 Orbtronics                     *
'*          : All Rights Reserved                               *
'*  Date    : 04-Oct-09                                         *
'*  Version : 2.0                                               *
'*  Notes   : USB MIDI Class Interface                          *
'*          : MIDI Streaming USB Demo                           *
'*          : Receiving MIDI Streaming data from a PC via USB   *
'*          : compiler version 3.4.0.7                          * 
;****************************************************************
;*     : Please read the:                                       *
;*     : Device Class Definition for Midi Devices               *
;*     : available from:                                        *
;*     : http://www.usb.org/developers/devclass_docs/midi10.pdf *
'****************************************************************                                                                                                                                                                                                                             
Device = 18F4550
Xtal = 48
Optimiser_Level = 3
 
All_Digital   = TRUE
PortB_Pullups = TRUE

Hserial_Baud = 115200		    ' Set baud rate to 115200
Hserial_RCSTA = %10010000       ' Enable serial port and continuous receive
Hserial_TXSTA = %00100100       ' Enable transmit and asynchronous mode 
Hserial_Clear = On		        ' Enable Error clearing on received characters

Symbol mLED_1   =  LATD.1
Symbol mLED_2   =  LATD.2
 Output PORTD.1
 Output PORTD.2
 Dim LED_COUNT As Word

' PIC Configuration Fuses

Config_Start
  PLLDIV = 5		; Divide by 5 (20 MHz input)
  CPUDIV = OSC1_PLL2; [OSC1/OSC2 Src: /1][96 MHz PLL Src: /2]
  USBDIV = 2		; Clock source from 96 MHz PLL/2
  FOSC = HSPLL_HS   ; HS oscillator, PLL enabled and under software control
  FCMEN = OFF		; Fail-Safe Clock Monitor Disabled
  IESO = OFF		; Internal/External Switch Over Disabled
  VREGEN = On		; USB Voltage Regulator Enabled
  WDT = OFF
  WDTPS = 128		; 1:128
  MCLRE = On		; Enabled
  LPT1OSC = On	    ; Timer1 oscillator configured for low power
  PBADEN = OFF	    ; PORTB<4:0> pins are configured as digital I/O on Reset
  STVREN = OFF	    ; Stack Overflow Reset Disabled
  LVP = OFF		    ; Low Voltage ICSP Disabled
  ICPRT = OFF		; Dedicated In-Circuit Debug/Programming Disabled
  XINST = OFF		; Extended Instruction Set Disabled
  CP0 = OFF		    ; Code Protection Block 0 Disabled
  CP1 = OFF		    ; Code Protection Block 1 Disabled
  CPB = OFF		    ; Boot Block Code Protection Disabled
  WRT0 = OFF		; Write Protection Block 0 Disabled
  WRT1 = OFF		; Write Protection Block 1 Disabled
  WRTB = OFF		; Boot Block Write Protection Disabled
  WRTC = OFF		; Configuration Register Write Protection Disabled
  EBTR0 = OFF		; Table Read Protection Block 0 Disabled
  EBTR1 = OFF		; Table Read Protection Block 1 Disabled
  EBTRB = OFF		; Boot Block Table Read Protection Disabled
Config_End		
	


USB_Descriptor  "MIDIDESC_3407.Inc"     ; USB Descriptor file

Include "USB_MD_MKR.Inc"                ; USB - Midi Macros/Subs file


ANY_VAR_INIT:
   UCFG.4 = 1                           ; UPUEN enable
   UCFG.2 = 1                           ; FSEN  enable
     
WAIT_USB_READY:                         ; Wait for PIC USB side to be ready
   Repeat								
      USBPoll		
   Until PP0 = CONFIGURED_STATE

MAIN_POLLING_LOOP:                      ; Main Loop
   While 1 = 1   
      GoSub Test_USB_Midi           
   Wend       

Test_USB_Midi:                        
               ;* Midi Receive example
               ;*----------------------------------------------------------------------------------------
               GoSub BlinkUSBStatus_SUB  ; show some action  for PICDEMFSUSB Board /// comment if not needed
               If STATUS.0 = 1 Then
                  USBIn 3, DATA_PACKET, Auto , Test_USB_Midi   ; save the payload in DATA_PACKET buffer
                  USB_PAK_LEN = PP2                            ; find the buffer length , though it is 64
                  For TEMP = 0 To USB_PAK_LEN - 1              ; as defined in descriptor file
                      HRSOut Hex2 DATA_PACKET[TEMP], " "       ; print the buffer to the serial port
                  Next                                         ; 
                  HRSOut 10,13                                 ; separation line
                EndIf
               ;*-----------------------------------------------------------------------------------------
               Return   
               
BlinkUSBStatus_SUB:                 ; show some action  for PICDEMFSUSB Board /// comment if not needed                
               If LED_COUNT = 0 Then 
                  LED_COUNT = 20000 
               EndIf
               Dec LED_COUNT;
               If UCON.1 = 1  Then
                   If LED_COUNT = 0 Then
                      mLED_1 = ~ mLED_1 
                      mLED_2 = mLED_1;        // Both blink at the same time
                   EndIf
                   Return       
               Else
                   Select PP0
                   Case DETACHED_STATE 
                      mLED_1 = 0
                      mLED_2 = 0 
                   Case ATTACHED_STATE 
                      mLED_1 = 1
                      mLED_2 = 1
                   Case POWERED_STATE 
                      mLED_1 = 1
                      mLED_2 = 0
                   Case DEFAULT_STATE   
                      mLED_1 = 0
                      mLED_2 = 1
                   Case ADDRESS_STATE
                       If LED_COUNT = 0 Then
                           mLED_1 = ~ mLED_1
                           mLED_2 = 0 ;
                       EndIf
                   Case CONFIGURED_STATE
                       If LED_COUNT = 0 Then
                           mLED_1 = ~ mLED_1  
                           mLED_2 = ~ mLED_1;       // Alternate blink                
                       EndIf
                   EndSelect
               EndIf
               Return                              
