'****************************************************************
'*  Name    : USB_MD_TX.BAS                                     *
'*  Author  : Gabi MIHAILA                                      *
'*  Notice  : Copyright (c) 2009 Orbtronics                     *
'*          : All Rights Reserved                               *
'*  Date    : 04-Oct-09                                         *
'*  Version : 2.0                                               *
'*  Notes   : USB MIDI Class Interface                          *
'*          : MIDI Streaming USB Demo                           *
'*          : Receiving MIDI Streaming data from a PC via USB   *
'*          : compiler version 3.4.0.7                          * 
;****************************************************************
;*     : Please read the:                                       *
;*     : Device Class Definition for Midi Devices               *
;*     : available from:                                        *
;*     : http://www.usb.org/developers/devclass_docs/midi10.pdf *
'****************************************************************                                                                                                                                                                                                                            
Device = 18F4550
Xtal = 48
Optimiser_Level = 3
 
All_Digital   = TRUE
PortB_Pullups = TRUE

' PIC Configuration Fuses

Config_Start
  PLLDIV = 5		; Divide by 5 (20 MHz input)
  CPUDIV = OSC1_PLL2; [OSC1/OSC2 Src: /1][96 MHz PLL Src: /2]
  USBDIV = 2		; Clock source from 96 MHz PLL/2
  FOSC = HSPLL_HS   ; HS oscillator, PLL enabled and under software control
  FCMEN = OFF		; Fail-Safe Clock Monitor Disabled
  IESO = OFF		; Internal/External Switch Over Disabled
  VREGEN = On		; USB Voltage Regulator Enabled
  WDT = OFF
  WDTPS = 128		; 1:128
  MCLRE = On		; Enabled
  LPT1OSC = On	    ; Timer1 oscillator configured for low power
  PBADEN = OFF	    ; PORTB<4:0> pins are configured as digital I/O on Reset
  STVREN = OFF	    ; Stack Overflow Reset Disabled
  LVP = OFF		    ; Low Voltage ICSP Disabled
  ICPRT = OFF		; Dedicated In-Circuit Debug/Programming Disabled
  XINST = OFF		; Extended Instruction Set Disabled
  CP0 = OFF		    ; Code Protection Block 0 Disabled
  CP1 = OFF		    ; Code Protection Block 1 Disabled
  CPB = OFF		    ; Boot Block Code Protection Disabled
  WRT0 = OFF		; Write Protection Block 0 Disabled
  WRT1 = OFF		; Write Protection Block 1 Disabled
  WRTB = OFF		; Boot Block Write Protection Disabled
  WRTC = OFF		; Configuration Register Write Protection Disabled
  EBTR0 = OFF		; Table Read Protection Block 0 Disabled
  EBTR1 = OFF		; Table Read Protection Block 1 Disabled
  EBTRB = OFF		; Boot Block Table Read Protection Disabled
Config_End		


USB_Descriptor  "MIDIDESC_3407.Inc"     ; USB Descriptor file

Include "USB_MD_MKR.Inc"                ; USB - Midi Macros/Subs file


ANY_VAR_INIT:
   UCFG.4 = 1                           ; UPUEN enable
   UCFG.2 = 1                           ; FSEN  enable
   Clear OUT_PACKET                     ; clear this buffer
     
WAIT_USB_READY:                         ; Wait for PIC USB side to be ready
   Repeat								
      USBPoll		
   Until PP0 = CONFIGURED_STATE

MAIN_POLLING_LOOP:                      ; Main Loop
   While 1 = 1   
      GoSub Test_USB_Midi
   Wend       

Test_USB_Midi: 
               ;* Midi Note Send Example 1
               ;*-------------------------------------------------------------------------------------------
               Note_ON 1, $30      ; C 3 : NoteON, on midi channel 1
               Note_ON 2, $34      ; E 3 : NoteON, on midi channel 2
               Note_ON 2, $37      ; G 3 : NoteON, on midi channel 2
               DelayMS 600         ; some demo pause
               Note_OFF 1, $30     ; C 3 : NoteOFF, on midi channel 1
               Note_OFF 2, $34     ; E 3 : NoteOFF, on midi channel 2
               Note_OFF 2, $37     ; G 3 : NoteOFF, on midi channel 2
               DelayMS 200         ; some demo pause
               ;*--------------------------------------------------------------------------------------------
               ;* Note: in real situation, those Note On or OFF above will be triggered after a keyboard scan,
               ;*     : keeping track of which note is in which position (ie On or OFF), and act accordingly.
               ;* Hint: For higher thru output, make use of the Code Index $04 Sysex Continue, and fill the
               ;*     : OUT_PACKET buffer as much as is possible.
               ;*--------------------------------------------------------------------------------------------
               
               ;* Midi Control Change Example
               ;*--------------------------------------------------------------------------------------------
               Control_Change 7, 91, 84   ; Set "Reverb Depth Control", on midi channel 7, to dec  84 value
               Control_Change 8, 7, 127   ; Set "Volume Control"      , on midi channel 8, to dec 127 value
               ;*--------------------------------------------------------------------------------------------
               ;* Note: in real situation, for a midi CC, midi controls will be sent upon changing their values,  
               ;*       like in a A/D chain scan that will send only the controls coresponding to the
               ;*       pots or sliders that have changed their previous value. 
               ;*-------------------------------------------------------------------------------------------
               Return
